package org.ylhealth.ym.springtest.hibernate;

import java.io.Serializable;
import javax.persistence.EntityManager;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.support.JpaRepositoryFactoryBean;
import org.springframework.data.repository.core.RepositoryInformation;
import org.springframework.data.repository.core.support.RepositoryFactorySupport;
import org.springframework.data.repository.core.support.RepositoryProxyPostProcessor;

/**
 * 將 Spring JPA 的讀取套上 i18n 轉換
 * @author scott.yang
 *
 * @param <R>
 * @param <T>
 * @param <I>
 */
public class I18NRepositoryFactoryBean<R extends JpaRepository<T, I>, T, I extends Serializable>
    extends JpaRepositoryFactoryBean<R, T, I> {

  public I18NRepositoryFactoryBean(Class<? extends R> repositoryInterface) {
    super(repositoryInterface);
  }

  @Override
  protected RepositoryFactorySupport createRepositoryFactory(EntityManager em) {
    RepositoryFactorySupport factory = super.createRepositoryFactory(em);
    factory.addRepositoryProxyPostProcessor(
        new RepositoryProxyPostProcessor() {
          @Override
          public void postProcess(
              ProxyFactory factory, RepositoryInformation repositoryInformation) {
            factory.addAdvice(new I18NPostProcessor(em));
          }
        });
    return factory;
  }
}
